
function [f,p]=dlqrsa(beta,A,B,Q,R,W);
%DLQRSA  Linear quadratic regulator design for discrete-time systems
%        when A of constraint equation is singular.
%
%	 [f,p] = dlqrsa(beta,A,B,Q,R,W)  calculates the optimal feedback
%        gain matrix F such that the feedback law:
%
%		u = -Fx
%
%        minimizes the cost function:
%
%		J = Sum beta^n{x[n]'Qx[n] + u[n]'Ru[n] + 2x[n]'Wu[n]}
%                    n
%
%        subject to the constraint equation:
%
%		x[n+1] = Ax[n] + Bu[n],  n=0,1,...
%
%	 Also returned is P, the steady-state solution to the associated
%	 discrete matrix Riccati equation:
%					                 -1
%		P = Q+beta A'PA-(beta A'PB+W)(R+beta B'PB)(beta BP'A+W')
 
%	 Ellen McGrattan 1-22-89
%	 Revised 5-15-90, ERM
 
[n,k]=size(B);
A = sqrt(beta)*(A-B/R*W');
B = sqrt(beta)*B;
Q = Q-W/R*W';
I = eye(n);
z = zeros(n);
m1 = [I,B/R*B';z,A'];
m2 = [A,z;-Q,I];
[v,d]=eig(m2,m1);
d = diag(d);
i = find(real(d)==NaN);
d(i) = inf*ones(length(i),1);
[d,index] = sort(abs(d));	 % sort on magnitude of eigenvalues
if (~((d(n) < 1) & (d(n+1) > 1)))
	error('Can''t order eigenvalues, (A,B) may be uncontrollable.')
end
% select vectors with eigenvalues inside unit circle
chi = v(1:n,index(1:n));
lambda = v((n+1):(2*n),index(1:n));
p = real(lambda/chi);
f = (R+B'*p*B)\B'*p*A+R\W';

