%  testrob3.m
%  modified version of recursea.m from ratdemo suite
%echo on
%cla
%
%
%
%
%
%
%     USING THE KALMAN FILTER TO SEASONALLY ADJUST
%
%
%     NOTE:  This demonstration takes several minutes, because
%     relative to "%classical" seasonal adjustment procedures, the
%     ones used here substitute brute force and the Kalman filter
%     for thought.  If you have a train to catch, kill this demo
%     by hitting "Ctrl,Break" and try another demo.
%
%
%     A reference for the techniques used here is Sargent's
%     "Linear Control, Filtering, and Rational Expectations."
%
%%pause    %Press a key to proceed with demonstration.
%cla

%     This program uses the Kalman filter to solve a "seasonal
%     adjustment" problem that comes in the form of a signal
%     extraction problem.
%
%     An observed process y(t) is the sum of three components:
%     a.) A "signal" f(t) that follows an autoregressive process
%
%         f(t) = a1(1)*f(t-1) + ... + a1(m)*y(t-m) + e1(t)
%
%     where e1(t) is a white noise with variance sig1.
%     b.) A "seasonal noise" s(t) that follows an a.r. process
%     
%         s(t) = a2(1)*s(t-1) + ... + a2(r)*s(t-r) + e2(t)
%
%     where e2(t) is a white noise with variance sig2.
%     c.) A "measurement error" e3(t) which is a white noise
%     with variance sig3.
%     NOTE:  To approximate the case in which y(t) = f(t) + s(t),
%     set sig3 equal to a very small positive number.
%     The goal is to compute the linear least squares estimate
%     
%       E[f(t) | y(t-1), y(t-2), . . . ].
%%pause    %Press a key to continue
%cla
%     We solve the problem by mapping the system into 
%     state space notation, namely,
%
%       x(t+1) = A * x(t) + e(t)
%         y(t) = C * x(t) + v(t)
%      
%     where x(t) is an (nx1) state vector and y(t) is a (kx1)
%     vector of observations ( in our example, k=1).  The vector
%     e(t) is an (nx1) vector white noise with covariance matrix
%     Ee(t)e(t)'=Q.  The vector v(t) is a (kx1) vector white noise
%     which is orthogonal to e(s) for all t and s, and which
%     has covariance matrix R.
%
%     For our example, the state vector x(t) will be given by
%     
%        x(t) = [f(t) f(t-1) ... f(t-m) s(t) s(t-1) ... s(t-r)]'
%
%     while y(t) is simply the scalar observed variable.
%
%%pause     %Press a key to set parameters of a.r. processes
%cla
a1=[.9 0 0 0 0]
a2=[0 0 0 .9]
%%pause    %Press a key to form A matrix of state space representation
%cla
A1=compn(a1);
A2=compn(a2);
[n,n1]=size(A1);[m,m1]=size(A2);
g1=zeros(n,m);
A=[A1,g1;g1',A2]
%pause   %Press a key to form C.
%cla
C=zeros(1,n+m);
C(1,1)=1;
C(1,n+1)=1
%pause     %Press a key to set variance parameters and form R and Q
%cla
Q=zeros(n+m);
Q(1,1)=1;
Q(n+1,n+1)=1
R=.0001;
%pause    %Press a key to continue demonstration
%cla
%     Now we'll use the Kalman filter to achieve the 
%     "innovations representation"
%
%       x^(t+1) = (A -K*C)*x^(t) + K*a(t)
%         y(t) = C*x^(t) + a(t)
%
%     where a(t) = E[y(t) | y(t-1), y(t-2), ... ],
%     and K is the "Kalman gain".  The process a(t) is the
%     is the "innovation" in the y(t) process and has variance
%     given by vara=C*S*C', where S is the covariance matrix
%     of x^(t+1) - x(t+1).  The variable x^(t+1) is the 
%     linear least squares projection
%
%        x^(t+1) = E[x(t+1) | y(t), y(t-1), ... ].
%pause    %Press a key to compute K and S using the Kalman filter.
%cla
sig=.00
H=zeros(1,9); H(1,1)=1;
[K0,S0,ST0,GW0]=rfilter(A,C,Q,R,H',sig);    %Working, please wait


sig=.01
H=zeros(1,9); H(1,1)=1;
[K1,S1,ST1,GW1]=rfilter(A,C,Q,R,H',sig);    %Working, please wait



K=K0;
%pause    %Press a key to see Kalman gain K
K
%pause    %Press a key to see state estimate covariance matrix
S=S0;
%pause    %Press a key to see variance of innovation to y.
vara=C*S*C'
%pause    %Press a key to continue demonstration
%cla
%     Notice that the innovations representation can be written
%
%       x^(t+1) = A*x^(t) + K*a(t)
%          y(t) = C*x^(t) + a(t)
%
%     This is equivalent with a Wold moving average representation
%     for y(t), which we can represent in the rational form
%
%       den1(L)y(t) = num1(L)a(t)
%
%pause    %Press a key to create num1 and den1.
[num1,den1]=ss2tf(A,K0,C,1,1);  %Working, please wait
%pause    %Press a key to see num1
num1
%pause    %Press a key to see den1
den1
%pause   %Press a key to continue demonstration
%cla
%     We now calculate the spectrum of the filter 
%     num1(L)/den1(L).
%
%pause   %Press a key to continue
close all
figure(1)
st='spectrum of y(t) with unit variance of a(t)'
sp=show(num1,den1,256,st);
%pause    %Press a key to continue
%%cla
%     Now we shall obtain the impulse response of x^(t+1) to
%     y(t).  We rewrite the innovations representation as
%
%          x^(t+1) = (A - K*C) x^(t) + Ky(t)
%            x^(t) = eye*x^(t) + zeros * y(t)
%
%     We'll form the appropriate matrices, then use ss2tf to get
%     a vector representation for x^(t+1) of the form
%
%        den(L)x^(t+1) = num(L) y(t)
%   
%     where den(L) is a scalar polynomial in the lag operator
%     and num(L) is a vector polynomial in the lag operator, with
%     as many rows as components of x^(t+1).
%
%     Notice that den(L) and num(L) contain all the information
%     that we need to form each component of E[x^(t+1)|y(t),...]
%     Note that the first several components of E[x^(t+1)|y(t),...]
%     are E[f(t+1)|y(t),...], E[f(t)|y(t),...], E[f(t-1)|y(t),...],
%     and so on.  The lower rows of E[x^(t+1)|y(t),...] thus 
%     correspond to finite two-sided seasonally adjusted series.
%pause    %Press a key to form num and den
%%cla
C1=  eye(n+m);D1=zeros(n+m,1);
[num,den]=ss2tf(A-K0*C,K0,C1,D1,1);%    Working, please wait
%pause    %Press a key to see num
num
%pause    %Press a key to see den
den
%pause   %Press a key to continue
%%cla
%     We now construct the impulse response of the first component
%     of x^(t+1) to an innovation in y(t).  This is a representation
%     for E[f(t+1)|y(t),...] in terms of a(t),a(t-1)....  Notice
%     from the innovations representation
%
%        x^(t+1) = A*x^(t) + K*a(t)
%
%     that the spectrum of the first component of x^(t+1) is
%     proportional to that of the first component of x(t+1).  
%     This follows from the whiteness of a(t).  
%pause    %Press a key to form the representation
num2=conv(num1,num(1,:));
den2=conv(den1,den);
%pause    %Press a key to see num2
num2
%pause    %Press a key to see den2
den2
%pause     %Press a key to plot spectrum of filter
figure(2)
st='spectrum of E[f(t+1)|y(t),y(t-1),...]'
sp1=show(num2,den2,256,st);
%pause    %Press key to continue
%cla
%     We now construct the impulse response of the nth component
%     of x^(t+1) (i.e. E[f(t-n+1)|y(t), y(t-1),...]) to
%     innovations a(t) in y(t).  Notice that this is a finitely
%     two sided signal extraction of f(t-n+1) based on past, 
%     present, and several future values of y(t).
%     Because of the two-sidedness, there can occur "dips"
%     in the spectral density of the seasonally adjusted
%     process.
num3=conv(num1,num(n,:));
%pause    %Press a key to plot spectrum
figure(3)
st='spectrum of E[f(t-n+1)|y(t),y(t-1),...]'
[sp3,ff]=show(num3,den2,256,st);
%pause   %Press a key to continue
%cla
figure(4)
mm=[sp1',sp3'];
st='spectra of one-sided and two-sided estimators of f(t)'
semilogy(ff,mm),title(st),%pause
%cla
%     Now we calculate the zeros of the numerator polynomial
%     in the representation 
%
%       den2(L)E[f(t-n+1)|y(t),...] = num3(L) a(t)
%
r3=roots(num3);  %   Working, please wait
%pause   %Press a key to see the roots
%cla
r3
%
%pause   %Notice location of roots relative to one.  Press key
%cla
%     NOTE:  We have calculated the zeros of num3(z^(-1)), which
%     are the reciprocals of the zeros of num3(z).  The
%     "invertibility" condition is that the zeros of num3(z) be
%     outside the unit circle,or that the zeros of num3(z^(-1))
%     be inside the unit circle.
%
%     It is possible for some of these zeros to be outside
%     the unit circle, reflecting the signal extraction version
%     of the "invertibility problem" in rational expectations 
%     models discussed by Hansen and Sargent (1980)>
%
%pause    %Press a key to return to menu
